
// CodeGen.java
// Andrew Davison, May 2010, ad@fivedots.coe.psu.ac.th

/* In the code generation phase, a tree of WidgetNodes is converted
   into a string of Java GUI code which is added to the program text read 
   from the TEMPLATE_FNM file. The resulting Java application is
   stored in <fnm>App.java

*/

import java.awt.*;
import java.io.*;
import java.util.*;
import java.text.*;


public class CodeGen
{
  private static final String TEMPLATE_FNM = "TemplateForGUI.txt";
  private static final int SPINNER_WIDTH = 10;    // used ro specify a JSpinner's width


  private WidgetNode widgetTree;
  private int varCounter = 0;   // used to generate variable names
  private ArrayList<String> groupNames; 
     /* this keeps a record of the generated ButtonGroup names */  



  public CodeGen(WidgetNode tree)
  {
    widgetTree = tree;
    groupNames = new ArrayList<String>();
  } 



  public void makeGUICode(String fnm)
  {
    // construct class name for the top-level JFrame
    String className = fnm.substring(0, 1).toUpperCase() + 
                       fnm.substring(1, fnm.lastIndexOf('.')) + "App";
    // System.out.println("Class name: " + className);

    StringBuilder codeStr = genPanel(widgetTree, "getContentPane()");   // generate the GUI code
          // getContentPane() is the container of the top-level JFrame

    saveCode(className, codeStr.toString().trim());
  }  // end of makeGUICode()




  private void saveCode(String className, String code)
  /* add GUI code to the template code, and save to a new file; also insert
     the class name and current date */
  {
    SimpleDateFormat formatter = new SimpleDateFormat("EEEE, d MMMMM yyyy, HH:mm");
    String dateNow = formatter.format( new Date() );

    String templateStr = loadFile(TEMPLATE_FNM);
    templateStr = templateStr.replaceAll("#NAME", className);
    templateStr = templateStr.replace("#DATE", dateNow);
    templateStr = templateStr.replace("#GUI_CODE", code);   // GUI code replaces #GUI_CODE 

    try {
      FileWriter out = new FileWriter( new File(className+".java"));
      out.write(templateStr);
      out.close();
      System.out.println("Wrote GUI code to " + className+".java");
	}
    catch(IOException e)
    {  System.out.println("Could not write GUI code to " + className+".java");  }
  }  // end of saveCode()



  private String loadFile(String fnm) 
  // load fnm as a long string
  {
    StringBuilder sb = new StringBuilder();
    BufferedReader br = null;
    try {
      br = new BufferedReader( new FileReader(fnm) );
      String line = null; 
      while (( line = br.readLine()) != null)
        sb.append(line + "\n");
    }
    catch (FileNotFoundException ex) {
      System.out.println(fnm + " not found");
      System.exit(1);
    }
    catch (IOException ex){
      System.out.println("Problem reading from " + fnm);
      System.exit(1);
    }
    finally {
      try {
        if (br != null)
          br.close();
      }
      catch (IOException ex) {}
    }
    return sb.toString();
  }  // end of loadFile()



  // ------------------------ GUI code generation -------------------------------------


  private StringBuilder genPanel(WidgetNode wn, String parentCon)
  // a container is represented by a JPanel, which is added to its parent Container
  {
    String conName = "panel__" + varCounter;
    varCounter++;

    // create JPanel
    StringBuilder codeStr = new StringBuilder("    JPanel " + conName + " = new JPanel();\n");
    codeStr.append( genContainer(wn, conName) );   

    // add panel to enclosing container
    codeStr.append("\n" + addToContainer(parentCon, wn, conName) );
    return codeStr;
  }  // end of genPanel()



  private StringBuilder genContainer(WidgetNode wn, String conName)
  /* a container widgetNode will contain data about the following grammar elements:
         CONTAINER  ::=  LAYOUT [ edged [ STRING ]] '{'  ( BORDER_POS ( CONTAINER | WIDGET ))+ '}'
  */
  {
    StringBuilder codeStr = genLayout(wn, conName);   // convert layout info to code

    String edgeTitle =  wn.getEdgeTitle();
    if (edgeTitle != null) {
      if (edgeTitle == "")    // border required with no label
        codeStr.append("    " + conName + ".setBorder( " +
               // "BorderFactory.createEtchedBorder());\n" );
               "BorderFactory.createLineBorder(Color.BLUE.brighter()));\n" );  // a light blue line
      else  // add in the border title
        codeStr.append("    " + conName + ".setBorder( " +
               "BorderFactory.createTitledBorder(\"" + edgeTitle + "\"));\n" );
    }

    // generate code for the container's children
    ArrayList<WidgetNode> childNodes = wn.getChildren();
    for(WidgetNode kids : childNodes)
      codeStr.append( "\n" + genWidget(kids, conName) );

    return codeStr;
  }  // end of genContainer()



  private String addToContainer(String parentCon, WidgetNode wn, String name)
  /* add the name component to the parentCon parent container, 
     maybe in a given Border layout position */
  {
    if (wn.getBorderPos() == WidgetNode.NO_POS) // border position not used
      return "    " + parentCon + ".add(" + name + ");\n";
    else  // border layout being used, so include position
      return "    " + parentCon + ".add(" + name + 
                        ", BorderLayout." + wn.getBorderPosName().toUpperCase() + ");\n";
  }  // end of addToContainer()





  private StringBuilder genLayout(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following 5 layout types
       LAYOUT := flow | grid rowNo colNo | border | vbox | hbox 
  */
  {
    switch(wn.getLayoutType()) {
      case WidgetNode.FLOW :
        return new StringBuilder("    "+conName+".setLayout( new FlowLayout());\n");
      case WidgetNode.GRID :
        return new StringBuilder("    "+conName+".setLayout( new GridLayout(" +
                                        wn.getRows() +", " + wn.getColumns() + "));\n");
      case WidgetNode.BORDER : 
        return new StringBuilder("    "+conName+".setLayout( new BorderLayout());\n");
      case WidgetNode.VBOX : 
        return new StringBuilder("    "+conName+".setLayout( new BoxLayout(" +
                                        conName+", BoxLayout.Y_AXIS));\n");
      case WidgetNode.HBOX : 
        return new StringBuilder("    "+conName+".setLayout( new BoxLayout(" +
                                        conName+", BoxLayout.X_AXIS));\n");
      default :
        System.out.println("Layout not found");
        return new StringBuilder();
    }
  }  // end of genLayout()




  private StringBuilder genWidget(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
      WIDGET  ::=  Label | Button | Radio | Check | TextField |
                   TextArea | PopDown | List | Spinner | 
                   hslider | vslider |
                   Panel | hspace | vspace | canvas
  */
  {
    switch(wn.getWidgetType()) {
      case WidgetNode.LABEL  : return genLabel(wn, conName);
      case WidgetNode.BUTTON : return genButton(wn, conName);
      case WidgetNode.RADIO : return genRadio(wn, conName);
      case WidgetNode.CHECKBOX : return genCheckbox(wn, conName);
      case WidgetNode.TEXTFIELD : return genTextfield(wn, conName);
      case WidgetNode.PASSWORD : return genPasswordfield(wn, conName);
      case WidgetNode.TEXTAREA : return genTextarea(wn, conName);
      case WidgetNode.POPDOWN : return genPopdown(wn, conName);       // will become a JComboBox
      case WidgetNode.LIST : return genList(wn, conName);
      case WidgetNode.SPINNER : return genSpinner(wn, conName);
      case WidgetNode.HSLIDER : return genSlider(wn, conName, true);  // horizontal space
      case WidgetNode.VSLIDER : return genSlider(wn, conName, false); // vertical
      case WidgetNode.PANEL : return genPanel(wn, conName);
      case WidgetNode.HSPACE : return genSpace(wn, conName, true);   // horizontal slider
      case WidgetNode.VSPACE : return genSpace(wn, conName, false);  // vertical
      case WidgetNode.CANVAS : return genCanvas(wn, conName);        // will become a JPanel
      default :
        System.out.println("Component not recognized");
        return new StringBuilder();
    }
  }  // end of genWidget()



  private StringBuilder genLabel(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
             LABEL := label [  '=' NAME ] STRING     
                        // the string may be a filename of an image file
     convert to a string of JLabel code
  */
  {
    String name = makeName(wn);
    StringBuilder codeStr = new StringBuilder();
    String data = wn.getData();
    if (!isFileName(data))   // label is a string
      codeStr.append("    JLabel " + name + " = new JLabel(\"" + data + "\");\n");
    else  // label is an image filename
      codeStr.append("    JLabel " + name + " = new JLabel( " +
                                   "new ImageIcon(\"" + data + "\"));\n");

    codeStr.append( addToContainer(conName, wn, name) );
    return codeStr;
  }  // end of genLabel()


  private String makeName(WidgetNode wn)
  // extract the widgetNode's name, or construct one from its widget type name
  {
    String name = wn.getName();
    if (name == null) {   // no name found; invent one using widget type name
      name = wn.getWidgetTypeName() + "__" + varCounter;   // new names end in "__<number>"
      varCounter++;
    }
    return name;
  }  // end of makeName()



  private boolean isFileName(String nm)
  // is nm a filename?
  {  return (nm.indexOf('.') != -1);   } 



  private StringBuilder genButton(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
            BUTTON := button [ '=' NAME ] STRING
     convert to a string of JButton code, including a listener
  */
  {
    String name = makeName(wn);
    String data = wn.getData();
    StringBuilder codeStr = new StringBuilder(
               "    JButton " + name + " = new JButton(\"" + data + "\");\n");

/*  // enclose button in a panel so it won't be resized
    String panelName = "panel__" + varCounter;
    varCounter++;
    codeStr.append("    JPanel " + panelName + " = new JPanel();\n");
    codeStr.append("    " + panelName + ".add(" + name + ");\n");
    codeStr.append( addToContainer(conName, wn, panelName) );
*/
    codeStr.append( addToContainer(conName, wn, name) );

    codeStr.append( genActionListener(name, data) );
    return codeStr;
  }  // end of genButton()



  private StringBuilder genRadio(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
              RADIO := radio [ '=' NAME ] STRING  [ on ] [ group = NAME ]
     convert to a string of JRadioButton code, including a listener 
     and perhaps a ButtonGroup
  */
  {
    String name = makeName(wn);
    String data = wn.getData();
    StringBuilder codeStr = new StringBuilder(
            "    JRadioButton " + name + " = new JRadioButton(\"" + data + 
                                                           "\", " + wn.isOn() + ");\n");
    // perhaps add this radio button to a button group
    String groupName = wn.getGroupName();
    if (groupName != null) {
      if (!isKnownGroup(groupName))   // insert declaration here
        codeStr.append("    ButtonGroup " + groupName + " = new ButtonGroup();\n");
      codeStr.append("    " + groupName + ".add(" + name + ");\n");
    }

    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genItemListener(name, data) );
    return codeStr;
  }  // end of genRadio()


  private boolean isKnownGroup(String groupName)
  // is groupName a known button group name?
  {
    for(String nm : groupNames) {
      if (nm.equals(groupName))
        return true;
    }
    groupNames.add(groupName);   // store for next time
    return false;
  }  // end of isKnownGroup()



  private StringBuilder genCheckbox(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
           CHECKBOX := checkbox [ '=' NAME ] STRING  [ on ]
     convert to a string of JCheckBox code, including a listener
  */
  {
    String name = makeName(wn);
    String data = wn.getData();
    StringBuilder codeStr = new StringBuilder(
                  "    JCheckBox " + name + " = new JCheckBox(\"" + data + 
                                                           "\", " + wn.isOn() + ");\n");
    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genItemListener(name, data) );
    return codeStr;
  }  // end of genCheckbox()



  private StringBuilder genTextfield(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
         TEXTFIELD := textfield [COLS] [ '=' NAME ] STRING
     convert to a string of JTextField code, including a listener
  */
  {
    String name = makeName(wn);
    StringBuilder codeStr = new StringBuilder(
                     "    JTextField " + name + " = new JTextField(\"" + wn.getData() + 
                                                    "\", " + wn.getColumns() + ");\n");
    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genTextActionListener(name) );
    return codeStr;
  }  // end of genTextfield()




  private StringBuilder genPasswordfield(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
         PASSWORD := password [COLS] [ '=' NAME ]
     convert to a string of JPasswordField code, including a listener
  */
  {
    String name = makeName(wn);
    StringBuilder codeStr = new StringBuilder(
               "    JPasswordField " + name + " = new JPasswordField(" + wn.getColumns() + ");\n");
    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genPasswordListnener(name) );
    return codeStr;
  }  // end of genPasswordfield()



  private StringBuilder genTextarea(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
          TEXTAREA := textarea [ROWS COLS] [ '=' NAME ] STRING
     convert to a string of JTextArea code, including a listener
  */
  {
    String name = makeName(wn);
    StringBuilder codeStr = new StringBuilder(
                "    JTextArea " + name + " = new JTextArea(\"" + wn.getData() + 
                                 "\", " + wn.getRows() + ", " + wn.getColumns()  + ");\n");
    codeStr.append("    " + name + ".setLineWrap(true);\n");   // wrap the text

    // enclose text area in a scrollpane
    String scrollName = "scroll__" + varCounter;
    varCounter++;
    codeStr.append("    JScrollPane " + scrollName + " = new JScrollPane(" + name + ");\n");
          // ",JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);\n");
    codeStr.append( addToContainer(conName, wn, scrollName) );
    codeStr.append( genDocListener(name) );
    return codeStr;
  }  // end of genTextarea()




  private StringBuilder genPopdown(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
         POPDOWN := popdown [ '=' NAME ] '{' STRING+ '}'
     convert to a string of JComboBox code, including a listener
  */
  {
    String name = makeName(wn);

    // create items array for filling the popdown list
    String itemsName = "popdownItems__" + varCounter;
    varCounter++;
    StringBuilder codeStr = new StringBuilder( genItemsArray(itemsName, wn.getParams()) );

    // create JComboBox using items array
    codeStr.append("    JComboBox " + name + " = new JComboBox(" + itemsName + ");\n");
    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genPopDownListener(name) );
    return codeStr;
  }  // end of genPopdown()



  private StringBuilder genItemsArray(String name, ArrayList<String> params)
  // convert list of parameters into an array of item strings
  {
    StringBuilder arrayStr = new StringBuilder();
    arrayStr.append("    String[] " + name + " = {\n");
    arrayStr.append("        ");
    for(int i=0; i < params.size()-1; i++)
      arrayStr.append("\""+params.get(i)+"\", ");
    arrayStr.append("\""+ params.get( params.size()-1) + "\" };\n");
    return arrayStr;
  }  // end of genItemsArray()




  private StringBuilder genList(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
         LIST := list [ '=' NAME ] '{' STRING+ '}'
     convert to a string of JList code, including a listener
  */
  {
    String name = makeName(wn);

    // create items array for filling the list
    String itemsName = "listItems__" + varCounter;
    varCounter++;
    StringBuilder codeStr = new StringBuilder( genItemsArray(itemsName, wn.getParams()) );

    // create JList using items array
    codeStr.append("    JList " + name + " = new JList(" + itemsName + ");\n");
    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genListListener(name) );
    return codeStr;
  }  // end of genList()




  private StringBuilder genSpinner(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
          SPINNER := spinner [ '=' NAME ] '{' STRING+ '}'
     convert to a string of JSpinner code, including a listener
  */
  {
    String name = makeName(wn);

    // create items array for filling the spinner
    String itemsName = "spinnerItems__" + varCounter;
    varCounter++;
    StringBuilder codeStr = new StringBuilder( genItemsArray(itemsName, wn.getParams()) );

    // create JSpinner using items array
    String modelName = "spinModel__" + varCounter;
    varCounter++;
    codeStr.append("    final SpinnerListModel " + modelName + 
                                   " = new SpinnerListModel(" + itemsName + ");\n");  // create model
    codeStr.append("    JSpinner " + name + " = new JSpinner(" + modelName + ");\n");

    // set width of spinner field by accessing its editor's text field
    String editorName = "editor__" + varCounter;
    varCounter++;
    String fieldName = "textfield__" + varCounter;
    varCounter++;
    codeStr.append("    JComponent " + editorName + " = " + name + ".getEditor();\n");  // get editor
    codeStr.append("    JTextField " + fieldName + 
                              " = ((JSpinner.DefaultEditor)" + editorName + ").getTextField();\n");
    codeStr.append("    if (" + fieldName + " != null)\n");    // set text field width (if possible)
    codeStr.append("      " + fieldName + ".setColumns(" + SPINNER_WIDTH + ");\n");

    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genSpinnerListener(modelName) );
    return codeStr;
  }  // end of genSpinner()



  private StringBuilder genSlider(WidgetNode wn, String conName, boolean isHorizontal)
  /* the widgetNode will contain data about the following grammar elements:
        SLIDER := slider [up] [ '=' NAME ] 
     convert to a string of JSlider code, including a listener
  */
  {
    String name = "slider__" + varCounter;
    varCounter++;

    // create a horizontal or vertical slider
    StringBuilder codeStr = new StringBuilder();
    if (isHorizontal)
      codeStr.append("    JSlider " + name + " = new JSlider();\n");
    else
      codeStr.append("    JSlider " + name + " = new JSlider(JSlider.VERTICAL);\n");

    codeStr.append( addToContainer(conName, wn, name) );
    codeStr.append( genSliderListener(name) );
    return codeStr;
  }  // end of genSlider





  private StringBuilder genSpace(WidgetNode wn, String conName, boolean isHorizontal)
  /* the widgetNode will contain data about the following grammar elements:
         SPACE := space [ NUM_SPACES ] 
     convert space into a horizontal or vetical Box strut depending on the orientation
     of the parent container (which depends on its layout manager)
  */
  {
    String name = "space__" + varCounter;
    varCounter++;

    // create a Box strut of specified size
    int numSpaces = wn.getNumSpaces(); 
    StringBuilder codeStr = new StringBuilder();
    if (isHorizontal)   // 
      codeStr.append("    Component " + name + " = Box.createHorizontalStrut(" + numSpaces + ");\n");
    else
      codeStr.append("    Component " + name + " = Box.createVerticalStrut(" + numSpaces + ");\n");

    codeStr.append( addToContainer(conName, wn, name) );
    // no listener for a space
    return codeStr;
  }  // end of genSpace




  private StringBuilder genCanvas(WidgetNode wn, String conName)
  /* the widgetNode will contain data about the following grammar elements:
          CANVAS := canvas [ROWS COLS] [ '=' NAME ]
     translated into a specialized JPanel called CanvasPanel which handles and reports 
     mouse events; the panel has a specified size
  */
  {
    String name = makeName(wn);
    StringBuilder codeStr = new StringBuilder(
                 "    CanvasPanel " + name + " = new CanvasPanel();\n");
    codeStr.append("    " + name + ".setPreferredSize( new Dimension(" +        // set dimensions
                                            wn.getColumns() + ", " + wn.getRows()  + "));\n");
    codeStr.append( addToContainer(conName, wn, name) );
    // no listener here, since it's part of CanvasPanels
    return codeStr;
  }  // end of genCanvas()


  // ----------------------- listener generation code --------------------


  private String genActionListener(String name, String data)
  // action listener used by the JButton code
  {
    return "    "+name+ ".addActionListener( new ActionListener() {\n" +
           "       public void actionPerformed(ActionEvent e)\n" +
           "       {  System.out.println(\"Pressed \\\"" + data + "\\\" \");  }\n" +
           "    });\n";
  }  // end of genActionListener()



  private String genItemListener(String name, String data)
  // item listener used by the JCheckBox and JRadioButton code
  {
    return "    "+name+ ".addItemListener( new ItemListener() {\n" +
           "       public void itemStateChanged(ItemEvent e)\n" +
           "       { if (e.getStateChange() == ItemEvent.SELECTED)\n" +
           "           System.out.println(\"Selected \\\"" + data + "\\\" \");\n" +
           "         else\n" +
           "           System.out.println(\"Deselected \\\"" + data + "\\\" \");\n" +
           "       }\n" +
           "    });\n";
  }  // end of genItemListener()



  private String genTextActionListener(String name)
  // action listener used by the JTextField code
  {
    return "    "+name+ ".addActionListener( new ActionListener() {\n" +
           "       public void actionPerformed(ActionEvent e)\n" +
           "       {  System.out.println(\"Entered \" + e.getActionCommand() );  }\n" +
           "    });\n";
  }  // end of genTextActionListener()



  private String genPasswordListnener(String name)
  // action listener used by the JPasswordField code
  {
    return "    "+name+ ".addActionListener( new ActionListener() {\n" +
           "       public void actionPerformed(ActionEvent e)\n" +
           "       {  JPasswordField pwf = (JPasswordField) e.getSource();\n" +
           "          System.out.println(\"Entered \" + new String(pwf.getPassword()) );  }\n" +
           "    });\n";
  }  // end of genPasswordListnener()



  private String genPopDownListener(String name)
  // action listener used by the JComboBox code
  {
    return "    "+name+ ".addActionListener( new ActionListener() {\n" +
           "       public void actionPerformed(ActionEvent e)\n" +
           "       { JComboBox list = (JComboBox)e.getSource();\n" + 
           "         String itemName = (String)list.getSelectedItem();\n" +
           "         System.out.println(\"Selected \\\"\" + itemName + \"\\\" \");\n" +
           "       }\n" +
           "    });\n";
  }  // end of genPopDownListener()



  private String genListListener(String name)
  // list selection listener used by the JList code
  {
    return "    "+name+ ".addListSelectionListener( new ListSelectionListener() {\n" +
           "       public void valueChanged(ListSelectionEvent e)\n" +
           "       { JList list = (JList)e.getSource();\n" + 
           "         Object[] sels = list.getSelectedValues();\n" +    // multiple selections are possible
           "         if (!e.getValueIsAdjusting()) {\n" +
           "           System.out.println(\"-- list " + name + " selection --\");\n" +
           "           for (int i = 0; i < sels.length; i++)\n" +
           "              System.out.println(\"  \" + sels[i]);\n" +
           "         }\n" +
           "       }\n" +
           "    });\n";
  }  // end of genListListener()



  private String genDocListener(String name)
  // document selection listener used by the JTextArea code
  {
    return "    "+name+ ".getDocument().addDocumentListener( new DocumentListener() {\n" +
           "       public void changedUpdate(DocumentEvent e)\n" +
           "       {  System.out.println(\"Changed " + name + 
                               " at \" + e.getOffset() );  }\n" +
           "       public void insertUpdate(DocumentEvent e)\n" +
           "       {  System.out.println(\"Inserted into " + name + 
                               " at \" + e.getOffset() );  }\n" +
           "       public void removeUpdate(DocumentEvent e)\n" +
           "       {  System.out.println(\"Removed from " + name + 
                               " at \" + e.getOffset() );  }\n" +

           "    });\n";
  }  // end of genDocListener()



  private String genSpinnerListener(String name)
  // change listener used by the JSpinner code
  {
    return "    "+name+ ".addChangeListener( new ChangeListener() {\n" +
        // name is a finalized SpinnerListModel variable name
           "       public void stateChanged(ChangeEvent e)\n" +
           "       {  System.out.println(\"Selected \" + " + name + ".getValue() );  }\n" +
           "    });\n";
  }  // end of genSpinnerListener()



  private String genSliderListener(String name)
  // change listener used by the JSlider code
  {
    return "    "+name+ ".addChangeListener( new ChangeListener() {\n" +
           "       public void stateChanged(ChangeEvent e)\n" +
           "       {  JSlider slider = (JSlider)e.getSource();\n" + 
           "          System.out.println(\"Moved " + name + " to \" + slider.getValue() );  }\n" +
           "    });\n";
  }  // end of genSliderListener()




  private String genMouseListener(String name)
  // used to respond to clicks inside a 'canvas' (a JPanel)
  {
    return "    "+name+ ".addMouseListener( new MouseAdapter() {\n" +
           "       public void mousePressed( MouseEvent e)\n" +
           "       { System.out.println(\"Mouse pressed at (\" + e.getX() + \",\" + e.getY() + \") in "
                                                + name + "\");\n" +
           "       }\n" +
           "    });\n";
  }  // end of genMouseListener()


}  // end of CodeGen class


