
// Parser.java
// Andrew Davison, May 2010, ad@fivedots.coe.psu.ac.th

/* A standard recursive descent parser which uses the Lexer class to 
   read Token objects from an input file. The tokens are matched against
   a grammaar and a tree of WidgetNode objects is built.

   The grammar is:

     CONTAINER  ::=  LAYOUT [ edged [ STRING ]] '{'  ( BORDER_POS ( CONTAINER | WIDGET ))+ '}'

     LAYOUT := flow | grid rowNo colNo | border | vbox | hbox 
     BORDER_POS := ((north | east | south | west | center) ':' ) | epsilon

     WIDGET :=  LABEL | BUTTON | RADIO | CHECKBOX | TEXTFIELD | PASSWORD |
                TEXTAREA | POPDOWN | LIST | SPINNER | SLIDER | SPACE |
                CANVAS

     LABEL := label [ '=' NAME ] STRING
     BUTTON := button [ '=' NAME ] STRING
     RADIO := radio [ '=' NAME ] STRING  [ on ] [ group '=' NAME ]
     CHECKBOX := checkbox [ '=' NAME ] STRING  [ on ]
     TEXTFIELD := textfield [ COLS ] [ '=' NAME ] STRING
     PASSWORD := password [ COLS ] [ '=' NAME ]

     TEXTAREA := textarea [ROWS COLS] [ '=' NAME ] STRING
     POPDOWN := popdown [ '=' NAME ] '{' STRING+ '}'
     LIST := list [ '=' NAME ] '{' STRING+ '}'
     SPINNER := spinner [ '=' NAME ] '{' STRING+ '}'
     SLIDER := slider [up] [ '=' NAME ]
     SPACE := space [ NUM_SPACES ]

     CANVAS := canvas [ROWS COLS] [ '=' NAME ] 

     The methods that interface to the Lexer object are next() and two
     versions of match()
*/


import java.io.*;
import java.util.*;


public class Parser
{
  // default sizes
  private static final int NUM_SPACES = 5; // for spacers

  private static final int NUM_ROWS = 5;   // for textfields and textarea
  private static final int NUM_COLS = 10;

  private static final int NUM_PIXEL_ROWS = 100;   // pixel dimensions for canvas
  private static final int NUM_PIXEL_COLS = 150;


  private Lexer lexer;
  private Token token;


  public Parser(String fnm)
  {  lexer = new Lexer(fnm);   } 

    

  public WidgetNode parse()
  // this returns a tree of WidgetNodes
  {
    next();
    WidgetNode top = container(WidgetNode.NO_POS);   // top-level container has no border position

    if (token.getType() != Token.EOF)
      System.out.println("Input NOT recognized");
    return top;
  }  // end of parse()



  private WidgetNode container(int bdPosn)
  // CONTAINER  ::=  LAYOUT [ edged [ STRING ]] '{'  ( BORDER_POS ( CONTAINER | WIDGET ))+ '}'
  // bdPosn is the border position for this container, passed from its parent;
  // this is the only recursive rule in this grammar
  {
    WidgetNode wn = new WidgetNode(WidgetNode.PANEL, null, null, bdPosn);   // a panel
    layout(wn);

    if (token.isString("edged")) {
      match("edged");
      if (token.isString("{"))   // no label string after edged keyword
        wn.setEdgeTitle("");     // create a border with no title
      else {
        wn.setEdgeTitle( token.getText() );
        next();
      }
    }

    /* guess at whether a "space" child widget means horizontal or
       vertical space by recording the layout of its parent container
    */
    boolean isVerticallySpaced = false;
    if (wn.getLayoutType() == WidgetNode.VBOX)   // the only layout using a vertical space
      isVerticallySpaced = true;

    // we've reached '{'  ( BORDER_POS ( CONTAINER | WIDGET ))+ '}'
    match("{");
    do {
      int pos = getBorderPos();   // may be no border info (i.e. NO_POS)
      if ( WidgetNode.toLayoutType(token.getText()) != WidgetNode.NO_LAYOUT )  // this is a layout
        wn.addChild( container(pos) );   // so child is a container
      else   // child is a widget
        wn.addChild( widget(pos, isVerticallySpaced) );
    } while (!token.isString("}"));
    match("}");

    return wn;
  }  // end of container()




  private void layout(WidgetNode wn)
  // LAYOUT := flow | grid rowNo colNo | border | vbox | hbox 
  // only simple layout managers are supported
  {
    int layoutType = WidgetNode.toLayoutType( token.getText() );
    if (layoutType != WidgetNode.NO_LAYOUT) {
      wn.setLayoutType(layoutType);
      next();
      if (layoutType == WidgetNode.GRID) {
        wn.setRows( match(Token.NUMBER).getValue() );  // no. of rows
        wn.setColumns( match(Token.NUMBER).getValue());  // no. of cols
      }
    }
    else
      error(" in layout syntax");
  }  // end of layout()



  private int getBorderPos()
  // BORDER_POS := ((north | east | south | west | center) ':' ) | epsilon
  // epsilon means "nothing", which is returned as NO_POS
  {
    int pos = WidgetNode.toBorderPos( token.getText() );
    if (pos != WidgetNode.NO_POS) {
      next();
      match(":");
    }
    return pos;
  }  // end of getBorderPos()
  


  private WidgetNode widget(int pos, boolean isVerticallySpaced)
  // WIDGET :=  LABEL | BUTTON | RADIO | CHECKBOX | TEXTFIELD | PASSWORD |
  //            TEXTAREA | POPDOWN | LIST | SPINNER | SLIDER | SPACE
  {
    WidgetNode wn = null;
    if (token.isString("label"))
      wn = label(pos);
    else if (token.isString("button"))
      wn = button(pos);
    else if (token.isString("radio"))
      wn = radio(pos);
    else if (token.isString("checkbox"))
      wn = checkbox(pos);
    else if (token.isString("textfield"))
      wn = textfield(pos);
    else if (token.isString("password"))
      wn = password(pos);
    else if (token.isString("textarea"))
      wn = textarea(pos);
    else if (token.isString("popdown"))
      wn = popdown(pos);
    else if (token.isString("list"))
      wn = list(pos);
    else if (token.isString("spinner"))
      wn = spinner(pos);
    else if (token.isString("slider"))
      wn = slider(pos);
    else if (token.isString("space"))
      wn = space(pos, isVerticallySpaced);
    else if (token.isString("canvas"))
      wn = canvas(pos);
    else
      error(" in widget syntax");

    return wn;
  }  // end of widget()



  // ----------------- GUI widgets -----------------------

  /* the basic task here is to combine several pieces of Token information 
     into a WidgetNode which is added to a tree of those nodes */


  private WidgetNode label(int pos)
  // LABEL := label [ '=' NAME ] STRING
  // this will eventually become a JLabel
  {
    match("label");
    String nm = getName();
    String data = getData();
    return new WidgetNode(WidgetNode.LABEL, nm, data, pos);
  }  // end of label()



  private String getName()
  //  return name information from [ '=' NAME ], or null
  {
    String name = null;
    if (token.isString("=")) {
      match("=");
      if (token.getType() == Token.NAME) {
        name = token.getText();
        next();
      }
      else
        error (" -- no name found for widget");
    }
    return name;
  }  // end of getName()



  private String getData()
  // return STRING information
  {
    String data = null;
    if (token.getType() == Token.STRING) {
      data = token.getText();
      next();
    }
    else
      error(" -- no data string found");
    return data;
  }  // end of getData()



  private WidgetNode button(int pos)
  // BUTTON := button [ '=' NAME ] STRING
  // this will eventually become a JButton
  {
    match("button");
    String nm = getName();
    String data = getData();
    return new WidgetNode(WidgetNode.BUTTON, nm, data, pos);
  }  // end of button()



  private WidgetNode radio(int pos)
  // RADIO := radio [ '=' NAME ] STRING  [ on ] [ group '=' NAME ]
  // this will eventually become a JRadioBox 
  // if group is used then this will become a ButtonGroup at code generation time
 {
    match("radio");
    String nm = getName();
    String data = getData();
    WidgetNode wn = new WidgetNode(WidgetNode.RADIO, nm, data, pos);

    if (token.isString("on")) {
      wn.setOn(true);
      next();
    }
    if (token.isString("group")) {
      next();
      match("=");
      wn.setGroupName( match(Token.NAME).getText() );
    }
    return wn;
  }  // end of radio()



  private WidgetNode checkbox(int pos)
  // CHECKBOX := checkbox [ '=' NAME ] STRING  [ on ]
  // this will eventually become a JCheckbox
  {
    match("checkbox");
    String nm = getName();
    String data = getData();
    WidgetNode wn = new WidgetNode(WidgetNode.CHECKBOX, nm, data, pos);

    if (token.isString("on")) {
      wn.setOn(true);
      next();
    }
    return wn;
  }  // end of checkbox()



  private WidgetNode textfield(int pos)
  // TEXTFIELD := textfield [ COLS ] [ '=' NAME ] STRING
  // this will eventually become a JTextfield
  {
    match("textfield");
    int numCols = NUM_COLS;  // default width of the textfield
    if (token.getType() == Token.NUMBER) {
      numCols = token.getValue();
      next();
    }

    String nm = getName();
    String data = getData();
    WidgetNode wn = new WidgetNode(WidgetNode.TEXTFIELD, nm, data, pos);
    wn.setColumns(numCols);

    return wn;
  }  // end of textfield()



  private WidgetNode password(int pos)
  // PASSWORD := password [ COLS ] [ '=' NAME ]
  // this will eventually become a JPasswordField
  {
    match("password");
    int numCols = NUM_COLS;  // default width of the password field
    if (token.getType() == Token.NUMBER) {
      numCols = token.getValue();
      next();
    }

    String nm = getName();
    WidgetNode wn = new WidgetNode(WidgetNode.PASSWORD, nm, null, pos);
    wn.setColumns(numCols);

    return wn;
  }  // end of password()




  private WidgetNode textarea(int pos)
  // TEXTAREA := textarea [ROWS COLS] [ '=' NAME ] STRING
  // this will eventually become a JTextarea
  {
    match("textarea");

    int numRows = NUM_ROWS; // default row and column dimensions of the textarea
    int numCols = NUM_COLS;
    if (token.getType() == Token.NUMBER) {
      numRows = token.getValue();
      next();
      numCols = match(Token.NUMBER).getValue();
    }

    String nm = getName();
    String data = getData();
    WidgetNode wn = new WidgetNode(WidgetNode.TEXTAREA, nm, data, pos);
    wn.setRows(numRows);
    wn.setColumns(numCols);

    return wn;
  }  // end of textarea()



  private WidgetNode popdown(int pos)
  // POPDOWN := popdown [ '=' NAME ] '{' STRING+ '}'
  // this will eventually become a JComboBox
  {
    match("popdown");
    String nm = getName();
    WidgetNode wn = new WidgetNode(WidgetNode.POPDOWN, nm, null, pos);
    storeParams(wn);

    return wn;
  }  // end of popdown()


  private void storeParams(WidgetNode wn)
  //  add each string param from '{' STRING+ '}' to the WidgetNode
  {
    match("{");
    do {
      if (token.getType() == Token.STRING) {
        wn.addParam( token.getText());
        next();
      }
      else
        error(" -- no string found");
    } while (!token.isString("}"));
    match("}");
  }  // end of storeParams()



  private WidgetNode list(int pos)
  // LIST := list [ '=' NAME ] '{' STRING+ '}'
  // this will eventually become a JList
  {
    match("list");
    String nm = getName();
    WidgetNode wn = new WidgetNode(WidgetNode.LIST, nm, null, pos);
    storeParams(wn);

    return wn;
  }  // end of list()



  private WidgetNode spinner(int pos)
  // SPINNER := spinner [ '=' NAME ] '{' STRING+ '}'
  // this will eventually become a JSpinner
  {
    match("spinner");
    String nm = getName();
    WidgetNode wn = new WidgetNode(WidgetNode.SPINNER, nm, null, pos);
    storeParams(wn);

    return wn;
  }  // end of spinner()



  private WidgetNode slider(int pos)
  // SLIDER := slider [up] [ '=' NAME ] 
  // this will eventually become either a horizontal or vertical JSlider
  {
    match("slider");

    int sliderType = WidgetNode.HSLIDER;   // horizontal slider is the default
    if (token.isString("up")) {
      sliderType = WidgetNode.VSLIDER;  // change to vertical slider
      next();
    }
    String nm = getName();
    WidgetNode wn = new WidgetNode(sliderType, nm, null, pos);

    return wn;
  }  // end of slider()




  private WidgetNode space(int pos, boolean isVerticallySpaced)
  // SPACE := space [ NUM_SPACES ] 
  // this will eventually become a horizontal or vertical Box strut
  {
    match("space");
    int spacerType = (isVerticallySpaced) ? WidgetNode.VSPACE : WidgetNode.HSPACE;
    WidgetNode wn = new WidgetNode(spacerType, null, null, pos);  // vertical or horizontal space

    int numSpaces = NUM_SPACES;   // default size of the Box
    if (token.getType() == Token.NUMBER) {
      numSpaces = token.getValue();
      next();
    }
    wn.setNumSpaces(numSpaces);
    return wn;
  }  // end of space()



  private WidgetNode canvas(int pos)
  // CANVAS := canvas [ROWS COLS] [ '=' NAME ]
  // this will eventually become a JPanel
  {
    match("canvas");

    int numRows = NUM_PIXEL_ROWS; // default row and column *pixel* dimensions of the canvas
    int numCols = NUM_PIXEL_COLS;
    if (token.getType() == Token.NUMBER) {
      numRows = token.getValue();
      next();
      numCols = match(Token.NUMBER).getValue();
    }

    String nm = getName();
    WidgetNode wn = new WidgetNode(WidgetNode.CANVAS, nm, null, pos);
    wn.setRows(numRows);
    wn.setColumns(numCols);

    return wn;
  }  // end of canvas()


  // --------------- token matching ----------------------


  private void next()
  {  
    token = lexer.scan();  
    // System.out.println("next(): \"" + token + "\"");  // useful for debugging lexer
    if (token == null) {
      System.out.println("Missing token on line " + lexer.getLineNo());
      System.exit(1);
    }
  }  // end of next()
        

  private Token match(int type)
  // match using types
  {
    Token t = null;
    if (token.getType() == type) {
      t = token;
      next();
    }
    else
      error("; expecting " + Token.getTypeName(type));
    return t;
  }  // end of match()


  private Token match(String text)
  // match using text
  {
    Token t = null;
    if (token.isString(text)) {
      t = token;
      next();
    }
    else
      error("; expecting " + text);
    return t;
  }  // end of match()


  private void error(String msg)
  // after an error, just give up and die!
  {
    System.out.println("Incorrect token \"" + token + 
                          "\" on line " + token.getLineNo() + msg);
    System.exit(1);
  }  // end of error()


}  // end of Parser class
