
// WidgetNode.java
// Andrew Davison, May 2010, ad@fivedots.coe.psu.ac.th

/* A WidgetNode can represent a GUI control (e.g. a button, textfield) or a
   container (a panel with a specified layout). 

   The possible widget types don't correspond directly to Swing 
   components. For example, a Box strut for creating space is denoted
   by two types, hspace and vspace.

   Only the basic forms of layout are supported -- since the layout types
   below.

*/

import java.io.*;
import java.util.*;


public class WidgetNode
{
  // widget names
  private static final String[] WIDGET_NAMES = new String[] {
    "panel", "label", "button", "radio", "checkbox", "hspace", "vspace",
    "textfield", "textarea", "popdown", "list", "spinner", "password",
    "hslider", "vslider", "canvas"
  };

  // widget types
  public static final int NOT_KNOWN = -1;
  public static final int PANEL = 0;
  public static final int LABEL = 1;
  public static final int BUTTON = 2;
  public static final int RADIO = 3;
  public static final int CHECKBOX = 4;
  public static final int HSPACE = 5;    // hoizontal and vertical space
  public static final int VSPACE = 6;
  public static final int TEXTFIELD = 7;
  public static final int TEXTAREA = 8;
  public static final int POPDOWN = 9;
  public static final int LIST = 10;
  public static final int SPINNER = 11;
  public static final int PASSWORD = 12;
  public static final int HSLIDER = 13;  // horizontal and vertical slider
  public static final int VSLIDER = 14;
  public static final int CANVAS = 15;   // will become a JPanel


  // layout names
  private static final String[] LAYOUT_NAMES = new String[] {
     "flow", "grid", "border", "vbox", "hbox"  
  };

  // layout types
  public static final int NO_LAYOUT = -1;
  public static final int FLOW = 0;
  public static final int GRID = 1;
  public static final int BORDER = 2;
  public static final int VBOX = 3;   // vertical and horizontal Box layout
  public static final int HBOX = 4;


  // position names for border layout
  private static final String[] BORDER_NAMES = new String[] {
     "north", "east", "south", "west", "center"  
  };

  // position integers for border layout
  public static final int NO_POS = -1;     // used when a widget does not have a border position
  public static final int NORTH = 0;
  public static final int EAST = 1;
  public static final int SOUTH = 2;
  public static final int WEST = 3;
  public static final int CENTER = 4;


  private int widgetType;
  private String name, data;
  private int borderPos;

  private String edgeTitle;      // used by PANEL (the container) -- the title for a container's border
  private boolean isOn;          // used by RADIO and CHECKBOX to denote them being switched on
  private int layoutType;        // used by PANEL to specify its layout manager
  private int rows, columns;     // used by TEXTFIELD, TEXTAREA, CANVAS
  private int numSpaces;         // used by HSPACE/VSPACE to specify the amount of space
  private ArrayList<String> listParams;  
               // used by POPDOWN, LIST, SPINNER to store the strings used in their list
  private String groupName;      // used by RADIO to store its ButtonGroup name

  private ArrayList<WidgetNode> children;   
         // used by PANEL so a container can contain widgets and more containers



  public WidgetNode(int type, String nm, String dt, int pos)
  {
    widgetType = type;
    name = nm;
    data = dt;
    borderPos = pos;   // will be assigned NO_POS if Border layout is not being used

    edgeTitle = null;
    isOn = false;
    layoutType = NO_LAYOUT;
    rows = -1;     // -1 means not used
    columns = -1;
    numSpaces = -1;
    listParams = null;
    groupName = null;

    children = null;
  }  // end of WidgetNode()


  // ------------ get methods ---------------------------

  public int getWidgetType()
  {  return widgetType;  }

  public String getWidgetTypeName()
  { if (widgetType == NOT_KNOWN)
      return null;
    else
      return WIDGET_NAMES[widgetType];
  }

  public String getName()
  {  return name; }

  public String getData()
  {  return data; }

  public int getBorderPos()
  {  return borderPos;  }


  public String getBorderPosName()
  { if (borderPos == NO_POS)
      return null;
    else
      return BORDER_NAMES[borderPos];
  }

  public String getEdgeTitle()
  {  return edgeTitle;  }

  public boolean isOn()
  {  return isOn;  }

  public int getLayoutType()
  {  return layoutType;  }

  public int getRows()
  {  return rows;  }

  public int getColumns()
  {  return columns;  }

  public int getNumSpaces()
  {  return numSpaces;  }

  public ArrayList<String> getParams()
  {  return listParams;  }

  public String getGroupName()
  {  return groupName; }


  public ArrayList<WidgetNode> getChildren()
  {  return children;  }


  // ------------ set methods ---------------------------

  public void setEdgeTitle(String el)
  {  edgeTitle = el;    }

  public void setOn(boolean b)
  {  isOn = b;  }


  public void setLayoutType(int type)
  {  layoutType = type;  }

  public void setRows(int n)
  {  rows = n;  }

  public void setColumns(int n)
  {  columns = n;  }

  public void setNumSpaces(int n)
  {  numSpaces = n;  }

  public void addParam(String param)
  {
    if (listParams == null)
      listParams = new ArrayList<String>();
    listParams.add(param);  
  }  // end of addParam()

  public void setGroupName(String gn)
  {  groupName = gn; }


  public void addChild(WidgetNode cn)
  {  
    if (cn == null)
      return;

    if (children == null)
      children = new ArrayList<WidgetNode>();
    children.add(cn);  
  }  // end of addChild()



  // ---------------------- printing ---------------------

  public void print()
  { print(0);  }


  public void print(int indent)
  // print this node, and any child nodes
  {
    if (name == null) {
      if (data == null)
        iprintln(indent,  WIDGET_NAMES[widgetType]);
      else
        iprintln(indent,  WIDGET_NAMES[widgetType] + " data = \"" + data + "\"");
    }
    else {   // there is a name
      if (data == null)
        iprintln(indent,  WIDGET_NAMES[widgetType] + " " + name);
      else
        iprintln(indent,  WIDGET_NAMES[widgetType] + " " + name+ " data = \"" + data + "\"");
    }

    if (layoutType != NO_LAYOUT)
      iprintln(indent+2, LAYOUT_NAMES[layoutType]);

    if (borderPos != NO_POS)
      iprintln(indent+2, BORDER_NAMES[borderPos]);

    if(edgeTitle != null) {
      if(edgeTitle == "")    // edged, but no title
        iprintln(indent+2, "is edged");
      else
        iprintln(indent+2, "is edged with title \"" + edgeTitle + "\"");
    }

    if(isOn)
      iprintln(indent+2, "is on");

    if (rows != -1)    // -1 means not used
      iprintln(indent+2, "no. of rows: " + rows);
    if (columns != -1)
      iprintln(indent+2, "no. of rows: " + columns);
    if (numSpaces != -1)
      iprintln(indent+2, "no. of spaces: " + numSpaces);

    if (groupName != null)
     iprintln(indent+2, "group = " + groupName);

    if (listParams != null)
     iprintln(indent+2, "list params = " + paramsStr(listParams));

    if (children != null)
      for(WidgetNode node: children)    // recursively print the children
        node.print(indent+2);   
  }  // end of print()



  private void iprintln(int indent, String s)
  // print indent spaces followed by the s string
  {
    for(int i=0; i < indent; i++)
      System.out.print(" ");
    System.out.println(s);
  }  // end of iprintln()



  private String paramsStr(ArrayList<String> listParams)
  // build a string containing all the parameers in the list
  {
    String params = null;
    if (listParams != null) {
      params = "( ";
      for(String p : listParams)
         params += "\"" + p + "\" ";
      params += "}";
    }
    return params;
  }  // end of paramsStr()


  // ----------------------- static methods -------------------------------


  public static int toLayoutType(String layoutName)
  // convert layout name to an integer
  {
    for(int i=0; i < LAYOUT_NAMES.length; i++)
      if (layoutName.equals(LAYOUT_NAMES[i]))
        return i;
    return NO_LAYOUT;
  }  // end of toLayoutType()


  public static int toBorderPos(String borderName)
  // convert border position name to an integer
  { 
    for(int i=0; i < BORDER_NAMES.length; i++)
      if (borderName.equals(BORDER_NAMES[i]))
        return i;
    return NO_POS;
  }  // end of toBorderPos()

}  // end of WidgetNode class
